@@ -1,6 +1,15 @@
-# Clustalw modules
+# This code is part of the Biopython distribution and governed by its
+# license.  Please see the LICENSE file that should have been included
+# as part of this package.
+"""Code for calling ClustalW and parsing its output (OBSOLETE).
+
+This module has been superseded by the Bio.AlignIO framework for
+alignment parsing, and the ClustalW command line wrapper in
+Bio.Align.Applications for calling the tool. These are both described
+in the current version of the Biopython Tutorial and Cookbook.
+This means Bio.Clustalw is now obsolete and is likely to be deprecated
+and eventually removed in future releases of Biopython.
 
-"""
 A set of classes to interact with the multiple alignment command
 line program clustalw. 
 
@@ -12,54 +21,93 @@ This requires clustalw available from:
 ftp://ftp-igbmc.u-strasbg.fr/pub/ClustalW/.
 
 functions:
+o read
 o parse_file
 o do_alignment
 
 classes:
 o ClustalAlignment
-o _AlignCreator
 o MultipleAlignCL"""
 
 # standard library
 import os
-import string
+import sys
+import subprocess
 
 # biopython
-from Bio.Seq import Seq
-from Bio.SeqRecord import SeqRecord
 from Bio import Alphabet
 from Bio.Alphabet import IUPAC
-import clustal_format
 from Bio.Align.Generic import Alignment
-
-# PyXML package
-from xml.sax import saxutils
-from xml.sax import handler
+from Bio.Application import _escape_filename
 
 def parse_file(file_name, alphabet = IUPAC.unambiguous_dna, debug_level = 0):
-    """Parse the given file into a clustal aligment object.
+    """Parse the given file into a clustal aligment object (OBSOLETE).
     
     Arguments:
     o file_name - The name of the file to parse.
     o alphabet - The type of alphabet to use for the alignment sequences.
     This should correspond to the type of information contained in the file.
     Defaults to be unambiguous_dna sequence.
+
+    There is a deprecated optional argument debug_level which has no effect.
+
+    This function is obsolete, and any new code should call Bio.AlignIO
+    instead. For example using Bio.Clustalw, you might have:
+
+    >>> from Bio import Clustalw
+    >>> from Bio import Alphabet
+    >>> filename = "Clustalw/protein.aln"
+    >>> alpha = Alphabet.Gapped(Alphabet.generic_protein)
+    >>> align = Clustalw.parse_file(filename, alphabet=alpha)
+    >>> print align.get_alignment_length()
+    411
+    >>> clustalw_string = str(align)
+
+    This becomes:
+
+    >>> from Bio import AlignIO
+    >>> from Bio import Alphabet
+    >>> filename = "Clustalw/protein.aln"
+    >>> alpha = Alphabet.Gapped(Alphabet.generic_protein)
+    >>> align = AlignIO.read(open(filename), "clustal", alphabet=alpha)
+    >>> print align.get_alignment_length()
+    411
+    >>> assert clustalw_string == align.format("clustal")
     """ 
-    align_handler = _AlignCreator(Alphabet.Gapped(alphabet))
 
-    parser = clustal_format.format.make_parser(debug_level)
-    parser.setContentHandler(align_handler)
-    parser.setErrorHandler(handler.ErrorHandler())
+    # Avoid code duplication by calling Bio.AlignIO to do this for us.
+    handle = open(file_name, 'r')
+    from Bio import AlignIO
+    generic_alignment = AlignIO.read(handle, "clustal")
+    handle.close()
 
-    to_parse = open(file_name, 'r')
-    parser.parseFile(to_parse)
-    to_parse.close()
+    #Force this generic alignment into a ClustalAlignment... nasty hack
+    if isinstance(alphabet, Alphabet.Gapped) :
+        alpha = alphabet
+    else :
+        alpha = Alphabet.Gapped(alphabet)
+    clustal_alignment = ClustalAlignment(alpha)
+    clustal_alignment._records = generic_alignment._records
+    for record in clustal_alignment._records :
+        record.seq.alphabet = alpha
 
-    return align_handler.align
+    try :
+        clustal_alignment._version = generic_alignment._version
+    except AttributeError :
+        #Missing the version, could be a 3rd party tool's output
+        pass
+
+    try :       
+        clustal_alignment._star_info = generic_alignment._star_info
+    except AttributeError :
+        #Missing the consensus, again, this is not always present
+        pass
+
+    return clustal_alignment
 
 def do_alignment(command_line, alphabet=None):
-    """Perform an alignment with the given command line.
-
+    """Perform an alignment with the given command line (OBSOLETE).
+    
     Arguments:
     o command_line - A command line object that can give out
     the command line we will input into clustalw.
@@ -70,12 +118,30 @@ def do_alignment(command_line, alphabet=None):
     Returns:
     o A clustal alignment object corresponding to the created alignment.
     If the alignment type was not a clustal object, None is returned.
+
+    This function (and the associated command line object) are now obsolete.
+    Please use the Bio.Align.Applications.ClustalwCommandline wrapper with
+    the Python subprocess module (and Bio.AlignIO for parsing) as described
+    in the tutorial.
     """
-    run_clust = os.popen(str(command_line))
-    value = run_clust.close()
+    #We don't need to supply any piped input, but we setup the
+    #standard input pipe anyway as a work around for a python
+    #bug if this is called from a Windows GUI program.  For
+    #details, see http://bugs.python.org/issue1124861
+    child_process = subprocess.Popen(str(command_line),
+                                     stdin=subprocess.PIPE,
+                                     stdout=subprocess.PIPE,
+                                     stderr=subprocess.PIPE,
+                                     shell=(sys.platform!="win32")
+                                     )
+    #Use .communicate as can get deadlocks with .wait(), see Bug 2804
+    child_process.communicate() #ignore the stdout and strerr data
+    value = child_process.returncode
 
     # check the return value for errors, as on 1.81 the return value
     # from Clustalw is actually helpful for figuring out errors
+    # TODO - Update this for new error codes using in clustalw 2
+
     # 1 => bad command line option
     if value == 1:
         raise ValueError("Bad command line option in the command: %s"
@@ -117,20 +183,23 @@ def do_alignment(command_line, alphabet=None):
 
 
 class ClustalAlignment(Alignment):
-    """Work with the clustal aligment format.
+    """Work with the clustal aligment format (OBSOLETE).
 
     This format is the default output from clustal -- these files normally
     have an extension of .aln.
+
+    This obsolete alignment object is a subclass of the more general alignment
+    object used in Bio.AlignIO. The old practical difference is here str(align)
+    would give the alignment as a string in clustal format, whereas in general
+    you must do align.format("clustal"), which supports other formats too.
     """
     # the default version to use if one isn't set
     DEFAULT_VERSION = '1.81'
     
     def __init__(self, alphabet = Alphabet.Gapped(IUPAC.ambiguous_dna)):
         Alignment.__init__(self, alphabet)
-
         # represent all of those stars in the aln output format
         self._star_info = ''
-        
         self._version = ''
 
     def __str__(self):
@@ -139,171 +208,15 @@ class ClustalAlignment(Alignment):
         The output produced from this should also be formatted in valid
         clustal format.
         """
-        # if the version isn't set, we need to use the default
-        if self._version == '':
-            self._version = self.DEFAULT_VERSION
-        
-        output = "CLUSTAL X (%s) multiple sequence alignment\n\n\n" % \
-                 self._version
+        return self.format("clustal")
 
-        cur_char = 0
-        max_length = len(self._records[0].seq)
-
-        # keep displaying sequences until we reach the end
-        while cur_char != max_length:
-            # calculate the number of sequences to show, which will
-            # be less if we are at the end of the sequence
-            if (cur_char + 50) > max_length:
-                show_num = max_length - cur_char
-            else:
-                show_num = 50
-
-            # go through all of the records and print out the sequences
-            # when we output, we do a nice 80 column output, although this
-            # may result in truncation of the ids.
-            for record in self._records:
-                line = record.description[0:30].ljust(36)
-                line = line + record.seq.data[cur_char:(cur_char + show_num)]
-                
-                output = output + line + "\n"
-
-            # now we need to print out the star info, if we've got it
-            if self._star_info != '':
-                output = output + (" " * 36) + \
-                     self._star_info[cur_char:(cur_char + show_num)] + "\n"
-
-            output = output + "\n"
-            cur_char = cur_char + show_num
-
-        # have a extra newline, so strip two off and add one before returning
-        return string.rstrip(output) + "\n"
-            
-
-    def _add_star_info(self, stars):
-        """Add all of the stars, which indicate consensus sequence.
-        """
-        self._star_info = stars
-
-    def _add_version(self, version):
-        """Add the version information about the clustal file being read.
-        """
-        self._version = version
-
-class _AlignCreator(handler.ContentHandler):
-    """Handler to create a ClustalAlignment object from clustal file info.
-
-    This handler is used to accept events coming from a Martel parsing
-    stream, and acts like a normal SAX handler.
-
-    After parsing, the alignment object created is available as the
-    align attribute of the class.
-    """
-    def __init__(self, alphabet):
-        """Create a new handler ready to deal with output from Martel parsing.
-
-        Arguments:
-        o alphabet - The alphabet to create all of the new sequences with.
-        """
-        self.align = ClustalAlignment(alphabet)
-
-        # store sequence info in a dictionary
-        self.all_info = {}
-        self.all_keys = []
-
-        # the current id we are working with
-        self.cur_id = None
-
-        # info so we know how big the ids and sequences are
-        self.id_size = 0
-        self.space_size = 0
-        self.seq_size = 0
-        
-        # flags so we can keep track of where we are during the parse
-        self.in_version = 0
-        self.in_stars = 0
-        self.in_seq_id = 0
-        self.in_space = 0
-        self.in_seq = 0
-        self.all_star_info = ''
-
-    def startElement(self, name, attrs):
-        """Check the various tags for the info we are interested in."""
-        if name == "version":
-            self.in_version = 1
-            self.version_info = ''
-        elif name == "seq_id":
-            self.in_seq_id = 1
-            self.seq_id_info = ''
-        elif name == "seq_space":
-            self.in_space = 1
-            self.space_info = ''
-        elif name == "seq_info":
-            self.in_seq = 1
-            self.seq_info = ''
-        elif name == "match_stars":
-            self.in_stars = 1
-            self.star_info = ''
-
-    def characters(self, content):
-        if self.in_version:
-            self.version_info = self.version_info + content
-        elif self.in_seq_id:
-            self.seq_id_info = self.seq_id_info + content
-        elif self.in_space:
-            self.space_info = self.space_info + content
-        elif self.in_seq:
-            self.seq_info = self.seq_info + content
-        elif self.in_stars:
-            self.star_info = self.star_info + content
-
-    def endElement(self, name):
-        if name == "version":
-            self.in_version = 0
-            self.align._add_version(string.strip(self.version_info))
-        elif name == "seq_id":
-            self.in_seq_id = 0
-            self.id_size = len(self.seq_id_info)
-            self.cur_id = self.seq_id_info
-        elif name == "seq_space":
-            self.in_space = 0
-            self.space_size = len(self.space_info)
-        elif name == "seq_info":
-            self.in_seq = 0
-            self.seq_size = len(self.seq_info)
-
-            # if the id is already there, add the sequence info
-            if self.cur_id in self.all_info.keys():
-                self.all_info[self.cur_id] = self.all_info[self.cur_id] + \
-                                             self.seq_info
-            else:
-                self.all_info[self.cur_id] = self.seq_info
-                self.all_keys.append(self.cur_id)
-                
-        elif name == "match_stars":
-            id_length = self.id_size + self.space_size
-            line_length = id_length + self.seq_size
-            
-            self.all_star_info = self.all_star_info + \
-                            self.star_info[id_length:line_length]
-                                                                           
-    def endDocument(self):
-        # when we are done parsing add all of the info we need
-        self.align._add_star_info(self.all_star_info)
-
-        for id in self.all_keys:
-            self.align.add_sequence(id, self.all_info[id])
-        
 class MultipleAlignCL:
-    """Represent a clustalw multiple alignment command line.
+    """Represent a clustalw multiple alignment command line (OBSOLETE).
 
-    This is meant to make it easy to code the command line options you
-    want to submit to clustalw.
-
-    Clustalw has a ton of options and things to do but this is set up to
-    represent a clustalw mutliple alignment.
-
-    Warning: I don't use all of these options personally, so if you find
-    one to be broken for any reason, please let us know!
+    This command line wrapper is considerd obsolete. Please use the replacement
+    Bio.Align.Applications.ClustalwCommandline wrapper instead, which uses the
+    standardised Bio.Application style interface. This is described in the
+    tutorial, with examples using ClustalW.
     """
     # set the valid options for different parameters
     OUTPUT_TYPES = ['GCG', 'GDE', 'PHYLIP', 'PIR', 'NEXUS', 'FASTA']
@@ -377,60 +290,107 @@ class MultipleAlignCL:
 
     def __str__(self):
         """Write out the command line as a string."""
-        cline = self.command + " " + self.sequence_file
+
+        #On Linux with clustalw 1.83, you can do:
+        #clustalw input.faa
+        #clustalw /full/path/input.faa
+        #clustalw -INFILE=input.faa
+        #clustalw -INFILE=/full/path/input.faa
+        #
+        #Note these fail (using DOS style slashes):
+        #
+        #clustalw /INFILE=input.faa
+        #clustalw /INFILE=/full/path/input.faa
+        #
+        #On Windows XP with clustalw.exe 1.83, these work at
+        #the command prompt:
+        #
+        #clustalw.exe input.faa
+        #clustalw.exe /INFILE=input.faa
+        #clustalw.exe /INFILE="input.faa"
+        #clustalw.exe /INFILE="with space.faa"
+        #clustalw.exe /INFILE=C:\full\path\input.faa
+        #clustalw.exe /INFILE="C:\full path\with spaces.faa"
+        #
+        #Sadly these fail:
+        #clustalw.exe "input.faa"
+        #clustalw.exe "with space.faa"
+        #clustalw.exe C:\full\path\input.faa
+        #clustalw.exe "C:\full path\with spaces.faa"
+        #
+        #Testing today (using a different binary of clustalw.exe 1.83),
+        #using -INFILE as follows seems to work.  However I had once noted:
+        #These also fail but a minus/dash does seem to
+        #work with other options (!):
+        #clustalw.exe -INFILE=input.faa
+        #clustalw.exe -INFILE=C:\full\path\input.faa
+        #
+        #Also these fail:
+        #clustalw.exe "/INFILE=input.faa"
+        #clustalw.exe "/INFILE=C:\full\path\input.faa"
+        #
+        #Thanks to Emanuel Hey for flagging this on the mailing list.
+        #
+        #In addition, both self.command and self.sequence_file
+        #may contain spaces, so should be quoted. But clustalw
+        #is fussy.
+        cline = _escape_filename(self.command)
+        cline += ' -INFILE=%s' % _escape_filename(self.sequence_file)
 
         # general options
         if self.type:
-            cline = cline + " -TYPE=%s" % self.type
+            cline += " -TYPE=%s" % self.type
         if self.is_quick == 1:
-            cline = cline + " -INTERACTIVE"
+            #Some versions of clustalw are case sensitive,
+            #and require -quicktree rather than -QUICKTREE
+            cline += " -quicktree"
         if self.allow_negative == 1:
-            cline = cline + " -NEGATIVE"
+            cline += " -NEGATIVE"
 
         # output options
         if self.output_file:
-            cline = cline + " -OUTFILE=%s" % self.output_file
+            cline += " -OUTFILE=%s" % _escape_filename(self.output_file)
         if self.output_type:
-            cline = cline + " -OUTPUT=%s" % self.output_type
+            cline += " -OUTPUT=%s" % self.output_type
         if self.output_order:
-            cline = cline + " -OUTORDER=%s" % self.output_order
+            cline += " -OUTORDER=%s" % self.output_order
         if self.change_case:
-            cline = cline + " -CASE=%s" % self.change_case
+            cline += " -CASE=%s" % self.change_case
         if self.add_seqnos:
-            cline = cline + " -SEQNOS=%s" % self.add_seqnos
+            cline += " -SEQNOS=%s" % self.add_seqnos
         if self.new_tree:
             # clustal does not work if -align is written -ALIGN
-            cline = cline + " -NEWTREE=%s -align" % self.new_tree
+            cline += " -NEWTREE=%s -align" % _escape_filename(self.new_tree)
 
         # multiple alignment options
         if self.guide_tree:
-            cline = cline + " -USETREE=%s" % self.guide_tree
+            cline += " -USETREE=%s" % _escape_filename(self.guide_tree)
         if self.protein_matrix:
-            cline = cline + " -MATRIX=%s" % self.protein_matrix
+            cline += " -MATRIX=%s" % self.protein_matrix
         if self.dna_matrix:
-            cline = cline + " -DNAMATRIX=%s" % self.dna_matrix
+            cline += " -DNAMATRIX=%s" % self.dna_matrix
         if self.gap_open_pen:
-            cline = cline + " -GAPOPEN=%s" % self.gap_open_pen
+            cline += " -GAPOPEN=%s" % self.gap_open_pen
         if self.gap_ext_pen:
-            cline = cline + " -GAPEXT=%s" % self.gap_ext_pen
+            cline += " -GAPEXT=%s" % self.gap_ext_pen
         if self.is_no_end_pen == 1:
-            cline = cline + " -ENDGAPS"
+            cline += " -ENDGAPS"
         if self.gap_sep_range:
-            cline = cline + " -GAPDIST=%s" % self.gap_sep_range
+            cline += " -GAPDIST=%s" % self.gap_sep_range
         if self.is_no_pgap == 1:
-            cline = cline + " -NOPGAP"
+            cline += " -NOPGAP"
         if self.is_no_hgap == 1:
-            cline = cline + " -NOHGAP"
+            cline += " -NOHGAP"
         if len(self.h_gap_residues) != 0:
             # stick the list of residues together as one big list o' residues
             residue_list = ''
             for residue in self.h_gap_residues:
                 residue_list = residue_list + residue
-            cline = cline + " -HGAPRESIDUES=%s" % residue_list
+            cline += " -HGAPRESIDUES=%s" % residue_list
         if self.max_div:
-            cline = cline + " -MAXDIV=%s" % self.max_div
+            cline += " -MAXDIV=%s" % self.max_div
         if self.trans_weight:
-            cline = cline + " -TRANSWEIGHT=%s" % self.trans_weight
+            cline += " -TRANSWEIGHT=%s" % self.trans_weight
 
         return cline
 
@@ -441,7 +401,7 @@ class MultipleAlignCL:
         self.output_file = output_file
 
         if output_type:
-            output_type = string.upper(output_type)
+            output_type = output_type.upper()
             if output_type not in self.OUTPUT_TYPES:
                 raise ValueError("Invalid output type %s. Valid choices are %s"
                                  % (output_type, self.OUTPUT_TYPES))
@@ -449,7 +409,7 @@ class MultipleAlignCL:
                 self.output_type = output_type
 
         if output_order:
-            output_order = string.upper(output_order)
+            output_order = output_order.upper()
             if output_order not in self.OUTPUT_ORDER:
                 raise ValueError("Invalid output order %s. Valid choices are %s"
                                  % (output_order, self.OUTPUT_ORDER))
@@ -457,7 +417,7 @@ class MultipleAlignCL:
                 self.output_order = output_order
 
         if change_case:
-            change_case = string.upper(change_case)
+            change_case = change_case.upper()
             if output_type != "GDE":
                 raise ValueError("Change case only valid for GDE output.")
             elif change_case not in self.CHANGE_CASE:
@@ -467,7 +427,7 @@ class MultipleAlignCL:
                 self.change_case = change_case
 
         if add_seqnos:
-            add_seqnos = string.upper(add_seqnos)
+            add_seqnos = add_seqnos.upper()
             if output_type:
                 raise ValueError("Add SeqNos only valid for CLUSTAL output.")
             elif add_seqnos not in self.OUTPUT_SEQNOS:
@@ -498,22 +458,21 @@ class MultipleAlignCL:
         Protein matrix can be either one of the defined types (blosum, pam,
         gonnet or id) or a file with your own defined matrix.
         """
-        if string.upper(protein_matrix) in self.PROTEIN_MATRIX:
-            self.protein_matrix = string.upper(protein_matrix)
+        if protein_matrix.upper() in self.PROTEIN_MATRIX:
+            self.protein_matrix = protein_matrix.upper()
         elif os.path.exists(protein_matrix):
             self.protein_matrix = protein_matrix
         else:
             raise ValueError("Invalid matrix %s. Options are %s or a file." %
-                             (string.upper(protein_matrix),
-                              self.PROTEIN_MATRIX))
+                             (protein_matrix.upper(), self.PROTEIN_MATRIX))
 
     def set_dna_matrix(self, dna_matrix):
         """Set the type of DNA matrix to use.
 
         The dna_matrix can either be one of the defined types (iub or clustalw)
         or a file with the matrix to use."""
-        if string.upper(dna_matrix) in self.DNA_MATRIX:
-            self.dna_matrix = string.upper(dna_matrix)
+        if dna_matrix.upper() in self.DNA_MATRIX:
+            self.dna_matrix = dna_matrix.upper()
         elif os.path.exists(dna_matrix):
             self.dna_matrix = dna_matrix
         else:
@@ -528,10 +487,29 @@ class MultipleAlignCL:
         protein or DNA you are working with, so this allows you to set it
         explicitly.
         """
-        residue_type = string.upper(residue_type)
+        residue_type = residue_type.upper()
         if residue_type in self.RESIDUE_TYPES:
             self.type = residue_type
         else:
             raise ValueError("Invalid residue type %s. Valid choices are %s"
                              % (residue_type, self.RESIDUE_TYPES))
 
+def _test():
+    """Run the Bio.Clustalw module's doctests (PRIVATE).
+
+    This will try and locate the unit tests directory, and run the doctests
+    from there in order that the relative paths used in the examples work.
+    """
+    import doctest
+    import os
+    if os.path.isdir(os.path.join("..","..","Tests")) :
+        print "Runing doctests..."
+        cur_dir = os.path.abspath(os.curdir)
+        os.chdir(os.path.join("..","..","Tests"))
+        doctest.testmod()
+        os.chdir(cur_dir)
+        del cur_dir
+        print "Done"
+
+if __name__ == "__main__":
+    _test()
