@@ -1,4 +1,4 @@
-# Copyright 2006-2008 by Peter Cock.  All rights reserved.
+# Copyright 2006-2009 by Peter Cock.  All rights reserved.
 # This code is part of the Biopython distribution and governed by its
 # license.  Please see the LICENSE file that should have been included
 # as part of this package.
@@ -31,30 +31,8 @@ Note that the parse() function will all invoke the relevant parser for the
 format with its default settings.  You may want more control, in which case
 you need to create a format specific sequence iterator directly.
 
-For non-interlaced files (e.g. Fasta, GenBank, EMBL) with multiple records
-using a sequence iterator can save you a lot of memory (RAM).  There is
-less benefit for interlaced file formats (e.g. most multiple alignment file
-formats).  However, an iterator only lets you access the records one by one.
-
-If you want random access to the records by number, turn this into a list:
-
-    >>> from Bio import SeqIO
-    >>> handle = open("Fasta/f002", "rU")
-    >>> records = list(SeqIO.parse(handle, "fasta"))
-    >>> handle.close()
-    >>> print records[1].id
-    gi|1348917|gb|G26685|G26685
-
-If you want random access to the records by a key such as the record id,
-turn the iterator into a dictionary:
-
-    >>> from Bio import SeqIO
-    >>> handle = open("Fasta/f002", "rU")
-    >>> record_dict = SeqIO.to_dict(SeqIO.parse(handle, "fasta"))
-    >>> handle.close()
-    >>> print len(record_dict["gi|1348917|gb|G26685|G26685"])
-    413
-
+Input - Single Records
+======================
 If you expect your file to contain one-and-only-one record, then we provide
 the following 'helper' function which will return a single SeqRecord, or
 raise an exception if there are no records or more than one record:
@@ -86,6 +64,52 @@ The above code will work as long as the file contains at least one record.
 Note that if there is more than one record, the remaining records will be
 silently ignored.
 
+
+Input - Multiple Records
+========================
+For non-interlaced files (e.g. Fasta, GenBank, EMBL) with multiple records
+using a sequence iterator can save you a lot of memory (RAM).  There is
+less benefit for interlaced file formats (e.g. most multiple alignment file
+formats).  However, an iterator only lets you access the records one by one.
+
+If you want random access to the records by number, turn this into a list:
+
+    >>> from Bio import SeqIO
+    >>> handle = open("Fasta/f002", "rU")
+    >>> records = list(SeqIO.parse(handle, "fasta"))
+    >>> handle.close()
+    >>> print records[1].id
+    gi|1348917|gb|G26685|G26685
+
+If you want random access to the records by a key such as the record id,
+turn the iterator into a dictionary:
+
+    >>> from Bio import SeqIO
+    >>> handle = open("Fasta/f002", "rU")
+    >>> record_dict = SeqIO.to_dict(SeqIO.parse(handle, "fasta"))
+    >>> handle.close()
+    >>> len(record_dict)
+    3
+    >>> print len(record_dict["gi|1348917|gb|G26685|G26685"])
+    413
+
+However, using list() or the to_dict() function will load all the records
+into memory at once, and therefore is not possible on very large files.
+Instead, for *some* file formats Bio.SeqIO provides an indexing approach
+providing dictionary like access to any record. For example,
+
+    >>> from Bio import SeqIO
+    >>> record_dict = SeqIO.index("Fasta/f002", "fasta")
+    >>> len(record_dict)
+    3
+    >>> print len(record_dict["gi|1348917|gb|G26685|G26685"])
+    413
+
+Many but not all of the supported input file formats can be indexed like
+this. For example "ace", "embl", "fasta", "fastq", "genbank", "ig", "phd",
+"pir", "tab" and "qual" work, but alignment formats like "phylip", "clustalw"
+and "nexus" will not.
+
 Input - Alignments
 ==================
 You can read in alignment files as Alignment objects using Bio.AlignIO.
@@ -133,6 +157,17 @@ For sequential files formats (e.g. fasta, genbank) each "record block" holds
 a single sequence.  For these files it would probably be safe to call
 write() multiple times.
 
+Conversion
+==========
+The Bio.SeqIO.convert(...) function allows an easy interface for simple
+file format conversions. Additionally, it may use file format specific
+optimisations so this should be the fastest way too.
+
+In general however, you can combine the Bio.SeqIO.parse(...) function with the
+Bio.SeqIO.write(...) function for sequence file conversion. Using generator
+expressions provides a memory efficient way to perform filtering or other
+extra operations as part of the process.
+
 File Formats
 ============
 When specifying the file format, use lowercase strings.  The same format
@@ -242,7 +277,7 @@ import QualityIO #FastQ and qual files
 
 
 #Convention for format names is "mainname-subtype" in lower case.
-#Please use the same names as BioPerl where possible.
+#Please use the same names as BioPerl or EMBOSS where possible.
 #
 #Note that this simple system copes with defining
 #multiple possible iterators for a given format/extension
@@ -277,6 +312,7 @@ _FormatToWriter ={"fasta" : FastaIO.FastaWriter,
                   "fastq-sanger" : QualityIO.FastqPhredWriter,
                   "fastq-solexa" : QualityIO.FastqSolexaWriter,
                   "fastq-illumina" : QualityIO.FastqIlluminaWriter,
+                  "phd" : PhdIO.PhdWriter,
                   "qual" : QualityIO.QualPhredWriter,
                   }
 
@@ -324,8 +360,9 @@ def write(sequences, handle, format) :
     else :
         raise ValueError("Unknown format '%s'" % format)
 
-    assert isinstance(count, int), "Internal error - the underlying writer " \
-           + " should have returned the record count, not %s" % repr(count)
+    assert isinstance(count, int), "Internal error - the underlying %s " \
+           "writer should have returned the record count, not %s" \
+           % (format, repr(count))
     return count
     
 def parse(handle, format, alphabet=None) :
@@ -379,7 +416,7 @@ def parse(handle, format, alphabet=None) :
     """
     #NOTE - The above docstring has some raw \n characters needed
     #for the StringIO example, hense the whole docstring is in raw
-    #string more (see the leading r before the opening quote).
+    #string mode (see the leading r before the opening quote).
     from Bio import AlignIO
 
     #Try and give helpful error messages:
@@ -406,8 +443,8 @@ def parse(handle, format, alphabet=None) :
             return _force_alphabet(iterator_generator(handle), alphabet)
     elif format in AlignIO._FormatToIterator :
         #Use Bio.AlignIO to read in the alignments
-        #TODO - Once we drop support for Python 2.3, this helper function can be
-        #replaced with a generator expression.
+        #TODO - Can this helper function can be replaced with a generator expression,
+        #or something from itertools?
         return _iterate_via_AlignIO(handle, format, alphabet)
     else :
         raise ValueError("Unknown format '%s'" % format)
@@ -496,16 +533,15 @@ def to_dict(sequences, key_function=None) :
 
      - sequences  - An iterator that returns SeqRecord objects,
                     or simply a list of SeqRecord objects.
-     - key_function - Optional function which when given a SeqRecord
-                      returns a unique string for the dictionary key.
+     - key_function - Optional callback function which when given a
+                    SeqRecord should return a unique key for the dictionary.
 
     e.g. key_function = lambda rec : rec.name
     or,  key_function = lambda rec : rec.description.split()[0]
 
-    If key_function is ommitted then record.id is used, on the
-    assumption that the records objects returned are SeqRecords
-    with a unique id field.
-
+    If key_function is ommitted then record.id is used, on the assumption
+    that the records objects returned are SeqRecords with a unique id.
+    
     If there are duplicate keys, an error is raised.
 
     Example usage, defaulting to using the record.id as key:
@@ -514,13 +550,13 @@ def to_dict(sequences, key_function=None) :
     >>> handle = open("GenBank/cor6_6.gb", "rU")
     >>> format = "genbank"
     >>> id_dict = SeqIO.to_dict(SeqIO.parse(handle, format))
-    >>> print id_dict.keys()
-    ['L31939.1', 'AJ237582.1', 'X62281.1', 'AF297471.1', 'X55053.1', 'M81224.1']
+    >>> print sorted(id_dict.keys())
+    ['AF297471.1', 'AJ237582.1', 'L31939.1', 'M81224.1', 'X55053.1', 'X62281.1']
     >>> print id_dict["L31939.1"].description
     Brassica rapa (clone bif72) kin mRNA, complete cds.
 
-    A more complex example, using the key_function argument in order to use
-    a sequence checksum as the dictionary key:
+    A more complex example, using the key_function argument in order to
+    use a sequence checksum as the dictionary key:
 
     >>> from Bio import SeqIO
     >>> from Bio.SeqUtils.CheckSum import seguid
@@ -528,14 +564,18 @@ def to_dict(sequences, key_function=None) :
     >>> format = "genbank"
     >>> seguid_dict = SeqIO.to_dict(SeqIO.parse(handle, format),
     ...               key_function = lambda rec : seguid(rec.seq))
-    >>> for key, record in seguid_dict.iteritems() :
+    >>> for key, record in sorted(seguid_dict.iteritems()) :
     ...     print key, record.id
-    SabZaA4V2eLE9/2Fm5FnyYy07J4 X55053.1
-    l7gjJFE6W/S1jJn5+1ASrUKW/FA X62281.1
     /wQvmrl87QWcm9llO4/efg23Vgg AJ237582.1
-    TtWsXo45S3ZclIBy4X/WJc39+CY M81224.1
-    uVEYeAQSV5EDQOnFoeMmVea+Oow AF297471.1
     BUg6YxXSKWEcFFH0L08JzaLGhQs L31939.1
+    SabZaA4V2eLE9/2Fm5FnyYy07J4 X55053.1
+    TtWsXo45S3ZclIBy4X/WJc39+CY M81224.1
+    l7gjJFE6W/S1jJn5+1ASrUKW/FA X62281.1
+    uVEYeAQSV5EDQOnFoeMmVea+Oow AF297471.1
+
+    This approach is not suitable for very large sets of sequences, as all
+    the SeqRecord objects are held in memory. Instead, consider using the
+    Bio.SeqIO.index() function (if it supports your particular file format).
     """    
     if key_function is None :
         key_function = lambda rec : rec.id
@@ -548,6 +588,119 @@ def to_dict(sequences, key_function=None) :
         d[key] = record
     return d
 
+def index(filename, format, alphabet=None, key_function=None) :
+    """Indexes a sequence file and returns a dictionary like object.
+
+     - filename - string giving name of file to be indexed
+     - format   - lower case string describing the file format
+     - alphabet - optional Alphabet object, useful when the sequence type
+                  cannot be automatically inferred from the file itself
+                  (e.g. format="fasta" or "tab")
+     - key_function - Optional callback function which when given a
+                  SeqRecord identifier string should return a unique
+                  key for the dictionary.
+    
+    This indexing function will return a dictionary like object, giving the
+    SeqRecord objects as values:
+
+    >>> from Bio import SeqIO
+    >>> records = SeqIO.index("Quality/example.fastq", "fastq")
+    >>> len(records)
+    3
+    >>> sorted(records.keys())
+    ['EAS54_6_R1_2_1_413_324', 'EAS54_6_R1_2_1_443_348', 'EAS54_6_R1_2_1_540_792']
+    >>> print records["EAS54_6_R1_2_1_540_792"].format("fasta")
+    >EAS54_6_R1_2_1_540_792
+    TTGGCAGGCCAAGGCCGATGGATCA
+    <BLANKLINE>
+    >>> "EAS54_6_R1_2_1_540_792" in records
+    True
+    >>> print records.get("Missing", None)
+    None
+
+    Note that this psuedo dictionary will not support all the methods of a
+    true Python dictionary, for example values() is not defined since this
+    would require loading all of the records into memory at once.
+
+    When you call the index function, it will scan through the file, noting
+    the location of each record. When you access a particular record via the
+    dictionary methods, the code will jump to the appropriate part of the
+    file and then parse that section into a SeqRecord.
+
+    Note that not all the input formats supported by Bio.SeqIO can be used
+    with this index function. It is designed to work only with sequential
+    file formats (e.g. "fasta", "gb", "fastq") and is not suitable for any
+    interlaced file format (e.g. alignment formats such as "clustal").
+
+    For small files, it may be more efficient to use an in memory Python
+    dictionary, e.g.
+
+    >>> from Bio import SeqIO
+    >>> records = SeqIO.to_dict(SeqIO.parse(open("Quality/example.fastq"), "fastq"))
+    >>> len(records)
+    3
+    >>> sorted(records.keys())
+    ['EAS54_6_R1_2_1_413_324', 'EAS54_6_R1_2_1_443_348', 'EAS54_6_R1_2_1_540_792']
+    >>> print records["EAS54_6_R1_2_1_540_792"].format("fasta")
+    >EAS54_6_R1_2_1_540_792
+    TTGGCAGGCCAAGGCCGATGGATCA
+    <BLANKLINE>
+
+    As with the to_dict() function, by default the id string of each record
+    is used as the key. You can specify a callback function to transform
+    this (the record identifier string) into your prefered key. For example:
+
+    >>> from Bio import SeqIO
+    >>> def make_tuple(identifier) :
+    ...     parts = identifier.split("_")
+    ...     return int(parts[-2]), int(parts[-1])
+    >>> records = SeqIO.index("Quality/example.fastq", "fastq",
+    ...                       key_function=make_tuple)
+    >>> len(records)
+    3
+    >>> sorted(records.keys())
+    [(413, 324), (443, 348), (540, 792)]
+    >>> print records[(540, 792)].format("fasta")
+    >EAS54_6_R1_2_1_540_792
+    TTGGCAGGCCAAGGCCGATGGATCA
+    <BLANKLINE>
+    >>> (540, 792) in records
+    True
+    >>> "EAS54_6_R1_2_1_540_792" in records
+    False
+    >>> print records.get("Missing", None)
+    None
+
+    Another common use case would be indexing an NCBI style FASTA file,
+    where you might want to extract the GI number from the FASTA identifer
+    to use as the dictionary key.
+
+    Notice that unlike the to_dict() function, here the key_function does
+    not get given the full SeqRecord to use to generate the key. Doing so
+    would impose a severe performance penalty as it would require the file
+    to be completely parsed while building the index. Right now this is
+    usually avoided.
+    """
+    #Try and give helpful error messages:
+    if not isinstance(filename, basestring) :
+        raise TypeError("Need a filename (not a handle)")
+    if not isinstance(format, basestring) :
+        raise TypeError("Need a string for the file format (lower case)")
+    if not format :
+        raise ValueError("Format required (lower case string)")
+    if format != format.lower() :
+        raise ValueError("Format string '%s' should be lower case" % format)
+    if alphabet is not None and not (isinstance(alphabet, Alphabet) or \
+                                     isinstance(alphabet, AlphabetEncoder)) :
+        raise ValueError("Invalid alphabet, %s" % repr(alphabet))
+
+    #Map the file format to a sequence iterator:    
+    import _index #Lazy import
+    try :
+        indexer = _index._FormatToIndexedDict[format]
+    except KeyError :
+        raise ValueError("Unsupported format '%s'" % format)
+    return indexer(filename, alphabet, key_function)
 
 def to_alignment(sequences, alphabet=None, strict=True) :
     """Returns a multiple sequence alignment (OBSOLETE).
@@ -635,6 +788,61 @@ def to_alignment(sequences, alphabet=None, strict=True) :
         #but which takes SeqRecord objects.  See also Bug 1944
         alignment._records.append(record)
     return alignment
+
+def convert(in_file, in_format, out_file, out_format, alphabet=None) :
+    """Convert between two sequence file formats, return number of records.
+
+     - in_file - an input handle or filename
+     - in_format - input file format, lower case string
+     - out_file - an output handle or filename
+     - out_format - output file format, lower case string
+     - alphabet - optional alphabet to assume
+
+    NOTE - If you provide an output filename, it will be opened which will
+    overwrite any existing file without warning. This may happen if even
+    the conversion is aborted (e.g. an invalid out_format name is given).
+
+    For example, going from a filename to a handle:
+
+    >>> from Bio import SeqIO
+    >>> from StringIO import StringIO
+    >>> handle = StringIO("")
+    >>> SeqIO.convert("Quality/example.fastq", "fastq", handle, "fasta")
+    3
+    >>> print handle.getvalue()
+    >EAS54_6_R1_2_1_413_324
+    CCCTTCTTGTCTTCAGCGTTTCTCC
+    >EAS54_6_R1_2_1_540_792
+    TTGGCAGGCCAAGGCCGATGGATCA
+    >EAS54_6_R1_2_1_443_348
+    GTTGCTTCTGGCGTGGGTGGGGGGG
+    <BLANKLINE>
+    """
+    #TODO - Add optimised versions of important conversions
+    #For now just off load the work to SeqIO parse/write    
+    if isinstance(in_file, basestring) :
+        in_handle = open(in_file, "rU")
+        in_close = True
+    else :
+        in_handle = in_file
+        in_close = False
+    #Don't open the output file until we've checked the input is OK?
+    if isinstance(out_file, basestring) :
+        out_handle = open(out_file, "w")
+        out_close = True
+    else :
+        out_handle = out_file
+        out_close = False
+    #This will check the arguments and issue error messages,
+    #after we have opened the file which is a shame.
+    from _convert import _handle_convert #Lazy import
+    count = _handle_convert(in_handle, in_format,
+                            out_handle, out_format,
+                            alphabet)
+    #Must now close any handles we opened
+    if in_close : in_handle.close()
+    if out_close : out_handle.close()
+    return count
            
 def _test():
     """Run the Bio.SeqIO module's doctests.
@@ -652,6 +860,14 @@ def _test():
         os.chdir(cur_dir)
         del cur_dir
         print "Done"
+    elif os.path.isdir(os.path.join("Tests", "Fasta")) :
+        print "Runing doctests..."
+        cur_dir = os.path.abspath(os.curdir)
+        os.chdir(os.path.join("Tests"))
+        doctest.testmod()
+        os.chdir(cur_dir)
+        del cur_dir
+        print "Done"
 
 if __name__ == "__main__":
     #Run the doctests
